@@ -8,16 +8,70 @@
 """
 from SoftLayer.consts import API_PUBLIC_ENDPOINT, API_PRIVATE_ENDPOINT, \
     USER_AGENT
-from SoftLayer.transport import make_api_call
+from SoftLayer.transport import make_xml_rpc_api_call
 from SoftLayer.exceptions import SoftLayerError
+from SoftLayer.deprecated import DeprecatedClientMixin
 import os
 
+
+__all__ = ['Client', 'BasicAuthentication', 'TokenAuthentication',
+           'API_PUBLIC_ENDPOINT', 'API_PRIVATE_ENDPOINT']
+
 API_USERNAME = None
 API_KEY = None
 API_BASE_URL = API_PUBLIC_ENDPOINT
+VALID_CALL_ARGS = set([
+    'id',
+    'mask',
+    'filter',
+    'headers',
+    'raw_headers',
+    'limit',
+    'offset',
+])
 
 
-class Client(object):
+class AuthenticationBase(object):
+    def get_headers(self):
+        raise NotImplementedError
+
+
+class TokenAuthentication(AuthenticationBase):
+    def __init__(self, user_id, auth_token):
+        self.user_id = user_id
+        self.auth_token = auth_token
+
+    def get_headers(self):
+        return {
+            'authenticate': {
+                'complexType': 'PortalLoginToken',
+                'userId': self.user_id,
+                'authToken': self.auth_token,
+            }
+        }
+
+    def __repr__(self):
+        return "<TokenAuthentication: %s %s>" % (self.user_id, self.auth_token)
+
+
+class BasicAuthentication(AuthenticationBase):
+    def __init__(self, username, api_key):
+        self.username = username
+        self.api_key = api_key
+
+    def get_headers(self):
+        return {
+            'authenticate': {
+                'username': self.username,
+                'apiKey': self.api_key,
+            }
+        }
+
+    def __repr__(self):
+        return "<BasicAuthentication: %s>" % (self.username)
+
+
+class Client(DeprecatedClientMixin, object):
     """ A SoftLayer API client.
 
     :param service_name: the name of the SoftLayer API service to query
@@ -32,7 +86,8 @@ class Client(object):
         Set this to API_PRIVATE_ENDPOINT to connect via SoftLayer's private
         network.
     :param integer timeout: timeout for API requests
-    :param boolean verbose: prints details about every HTTP request if true
+    :param auth: an object which responds to get_headers() to be inserted into
+        the xml-rpc headers. Example: `BasicAuthentication`
 
     Usage:
 
@@ -46,142 +101,48 @@ class Client(object):
     _prefix = "SoftLayer_"
 
     def __init__(self, service_name=None, id=None, username=None, api_key=None,
-                 endpoint_url=None, timeout=None, verbose=False):
+                 endpoint_url=None, timeout=None, auth=None):
         self._service_name = service_name
-        self.verbose = verbose
         self._headers = {}
         self._raw_headers = {}
 
-        self.username = username or API_USERNAME or \
-            os.environ.get('SL_USERNAME') or ''
-        self.api_key = api_key or API_KEY or os.environ.get('SL_API_KEY') or ''
-        self.set_authentication(self.username, self.api_key)
-
-        if id is not None:
-            self.set_init_parameter(int(id))
+        self.auth = auth
+        if self.auth is None:
+            username = username or API_USERNAME or \
+                os.environ.get('SL_USERNAME') or ''
+            api_key = api_key or API_KEY or os.environ.get('SL_API_KEY') or ''
+            if username and api_key:
+                self.auth = BasicAuthentication(username, api_key)
 
         self._endpoint_url = (endpoint_url or API_BASE_URL or
                               API_PUBLIC_ENDPOINT).rstrip('/')
         self.timeout = timeout
 
-    def add_raw_header(self, name, value):
-        """ Set HTTP headers for API calls.
+        super(Client, self).__init__(
+            service_name=service_name, id=id, username=username,
+            api_key=api_key, endpoint_url=endpoint_url, timeout=timeout,
+            auth=auth)
 
-        :param name: the header name
-        :param value: the header value
+    def authenticate_with_password(self, username, password,
+                                   security_question_id=None,
+                                   security_question_answer=None):
+        """ Performs Username/Password Authentication and gives back an auth
+            handler to use to create a client that uses token-based auth.
 
-        ..  deprecated:: 2.0.0
+        :param string username: your SoftLayer username
+        :param string password: your SoftLayer password
+        :param int security_question_id: The security question id to answer
+        :param string security_question_answer: The answer to the security
+                                                question
 
         """
-        self._raw_headers[name] = value
-
-    def add_header(self, name, value):
-        """ Set a SoftLayer API call header.
-
-        :param name: the header name
-        :param value: the header value
-
-        ..  deprecated:: 2.0.0
-
-        """
-        name = name.strip()
-        if name is None or name == '':
-            raise SoftLayerError('Please specify a header name.')
-
-        self._headers[name] = value
-
-    def remove_header(self, name):
-        """ Remove a SoftLayer API call header.
-
-        :param name: the header name
-
-        ..  deprecated:: 2.0.0
-
-        """
-        if name in self._headers:
-            del self._headers[name.strip()]
-
-    def set_authentication(self, username, api_key):
-        """ Set user and key to authenticate a SoftLayer API call.
-
-        Use this method if you wish to bypass the API_USER and API_KEY class
-        constants and set custom authentication per API call.
-
-        See https://manage.softlayer.com/Administrative/apiKeychain for more
-        information.
-
-        :param username: the username to authenticate with
-        :param api_key: the user's API key
-
-        ..  deprecated:: 2.0.0
-
-        """
-        self.add_header('authenticate', {
-            'username': username.strip(),
-            'apiKey': api_key.strip(),
-        })
-
-    def set_init_parameter(self, id):
-        """ Set an initialization parameter header.
-
-        Initialization parameters instantiate a SoftLayer API service object to
-        act upon during your API method call. For instance, if your account has
-        a server with ID number 1234, then setting an initialization parameter
-        of 1234 in the SoftLayer_Hardware_Server Service instructs the API to
-        act on server record 1234 in your method calls.
-
-        See http://sldn.softlayer.com/article/Using-Initialization-Parameters-SoftLayer-API  # NOQA
-        for more information.
-
-        :param id: the ID of the SoftLayer API object to instantiate
-
-        ..  deprecated:: 2.0.0
-
-        """
-        self.add_header(self._service_name + 'InitParameters', {
-            'id': int(id)
-        })
-
-    def set_object_mask(self, mask):
-        """ Set an object mask to a SoftLayer API call.
-
-        Use an object mask to retrieve data related your API call's result.
-        Object masks are skeleton objects, or strings that define nested
-        relational properties to retrieve along with an object's local
-        properties. See
-        http://sldn.softlayer.com/article/Using-Object-Masks-SoftLayer-API
-        for more information.
-
-        :param mask: the object mask you wish to define
-
-        ..  deprecated:: 2.0.0
-
-        """
-        header = 'SoftLayer_ObjectMask'
-
-        if isinstance(mask, dict):
-            header = '%sObjectMask' % self._service_name
-
-        self.add_header(header, {'mask': mask})
-
-    def set_result_limit(self, limit, offset=0):
-        """ Set a result limit on a SoftLayer API call.
-
-        Many SoftLayer API methods return a group of results. These methods
-        support a way to limit the number of results retrieved from the
-        SoftLayer API in a way akin to an SQL LIMIT statement.
-
-        :param limit: the number of results to limit a SoftLayer API call to
-        :param offset: An optional offset at which to begin a SoftLayer API
-                       call's returned result
-
-        ..  deprecated:: 2.0.0
-
-        """
-        self.add_header('resultLimit', {
-            'limit': int(limit),
-            'offset': int(offset)
-        })
+        res = self['User_Customer'].getPortalLoginToken(
+            username,
+            password,
+            security_question_id,
+            security_question_answer)
+        self.auth = TokenAuthentication(res['userId'], res['hash'])
+        return (res['userId'], res['hash'])
 
     def __getitem__(self, name):
         """ Get a SoftLayer Service.
@@ -189,6 +150,7 @@ class Client(object):
         :param name: The name of the service. E.G. Account
 
         Usage:
+            >>> client = SoftLayer.Client()
             >>> client['Account']
             <Service: Account>
 
@@ -207,30 +169,32 @@ class Client(object):
         :param service: the name of the SoftLayer API service
 
         Usage:
+            >>> client = SoftLayer.Client()
             >>> client['Account'].getVirtualGuests(mask="id", limit=10)
             [...]
 
         """
-        if kwargs.get('iter'):
+        if kwargs.pop('iter', False):
             return self.iter_call(service, method, *args, **kwargs)
 
+        invalid_kwargs = set(kwargs.keys()) - VALID_CALL_ARGS
+        if invalid_kwargs:
+            raise TypeError(
+                'Invalid keyword arguments: %s' % ','.join(invalid_kwargs))
+
         if not service.startswith(self._prefix):
             service = self._prefix + service
 
         objectid = kwargs.get('id')
         objectmask = kwargs.get('mask')
         objectfilter = kwargs.get('filter')
-        headers = kwargs.get('headers')
+        headers = kwargs.get('headers', {})
         raw_headers = kwargs.get('raw_headers')
         limit = kwargs.get('limit')
         offset = kwargs.get('offset', 0)
 
-        if headers is None:
-            headers = {
-                'authenticate': {
-                    'username': self.username,
-                    'apiKey': self.api_key,
-                }}
+        if not headers and self.auth:
+            headers = self.auth.get_headers()
 
         http_headers = {
             'User-Agent': USER_AGENT,
@@ -258,9 +222,10 @@ class Client(object):
                 'offset': int(offset)
             }
         uri = '/'.join([self._endpoint_url, service])
-        return make_api_call(uri, method, args, headers=headers,
-                             http_headers=http_headers, timeout=self.timeout,
-                             verbose=self.verbose)
+        return make_xml_rpc_api_call(uri, method, args,
+                                     headers=headers,
+                                     http_headers=http_headers,
+                                     timeout=self.timeout)
 
     __call__ = call
 
@@ -340,34 +305,9 @@ class Client(object):
 
         return {mheader: {'mask': objectmask}}
 
-    def __getattr__(self, name):
-        """ Attempt a SoftLayer API call.
-
-        Use this as a catch-all so users can call SoftLayer API methods
-        directly against their client object. If the property or method
-        relating to their client object doesn't exist then assume the user is
-        attempting a SoftLayer API call and return a simple function that makes
-        an XML-RPC call.
-
-        :param name: method name
-
-        ..  deprecated:: 2.0.0
-
-        """
-        if name in ["__name__", "__bases__"]:
-            raise AttributeError("'Obj' object has no attribute '%s'" % name)
-
-        def call_handler(*args, **kwargs):
-            if self._service_name is None:
-                raise SoftLayerError(
-                    "Service is not set on Client instance.")
-            kwargs['headers'] = self._headers
-            return self.call(self._service_name, name, *args, **kwargs)
-        return call_handler
-
     def __repr__(self):
-        return "<Client: endpoint=%s, user=%s>" \
-            % (self._endpoint_url, self.username)
+        return "<Client: endpoint=%s, user=%r>" \
+            % (self._endpoint_url, self.auth)
 
     __str__ = __repr__
 
