@@ -25,27 +25,12 @@ hostname or the ip address for a CCI.
 from os import linesep
 import os.path
 
-from SoftLayer.CCI import CCIManager
+from SoftLayer import CCIManager
 from SoftLayer.CLI import (
     CLIRunnable, Table, no_going_back, confirm, mb_to_gb, listing,
     FormattedItem)
 from SoftLayer.CLI.helpers import (
-    CLIAbort, ArgumentError, SequentialOutput,
-    NestedDict, blank)
-
-
-def resolve_id(cci_manager, identifier):
-    cci_ids = cci_manager.resolve_ids(identifier)
-
-    if len(cci_ids) == 0:
-        raise CLIAbort("Error: Unable to find CCI '%s'" % identifier)
-
-    if len(cci_ids) > 1:
-        raise CLIAbort(
-            "Error: Multiple CCIs found for '%s': %s" %
-            (identifier, ', '.join([str(_id) for _id in cci_ids])))
-
-    return cci_ids[0]
+    CLIAbort, ArgumentError, SequentialOutput, NestedDict, blank, resolve_id)
 
 
 class ListCCIs(CLIRunnable):
@@ -108,16 +93,17 @@ For more on filters see 'sl help filters'
         t.sortby = args.get('--sortby') or 'host'
 
         for guest in guests:
+            guest = NestedDict(guest)
             t.add_row([
                 guest['id'],
-                guest.get('datacenter', {}).get('name', 'unknown'),
+                guest['datacenter']['name'] or blank(),
                 guest['fullyQualifiedDomainName'],
                 guest['maxCpu'],
                 mb_to_gb(guest['maxMemory']),
-                guest.get('primaryIpAddress', blank()),
-                guest.get('primaryBackendIpAddress', blank()),
-                guest.get('activeTransaction', {}).get(
-                    'transactionStatus', {}).get('friendlyName', blank()),
+                guest['primaryIpAddress'] or blank(),
+                guest['primaryBackendIpAddress'] or blank(),
+                guest['activeTransaction']['transactionStatus'].get(
+                    'friendlyName') or blank(),
             ])
 
         return t
@@ -143,7 +129,7 @@ Options:
         t.align['Name'] = 'r'
         t.align['Value'] = 'l'
 
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         result = cci.get_instance(cci_id)
         result = NestedDict(result)
 
@@ -151,19 +137,18 @@ Options:
         t.add_row(['hostname', result['fullyQualifiedDomainName']])
         t.add_row(['status', result['status']['name']])
         t.add_row(['state', result['powerState']['name']])
-        t.add_row(['datacenter', result['datacenter'].get('name', blank())])
+        t.add_row(['datacenter', result['datacenter']['name'] or blank()])
         t.add_row(['cores', result['maxCpu']])
         t.add_row(['memory', mb_to_gb(result['maxMemory'])])
-        t.add_row(['public_ip', result.get('primaryIpAddress', blank())])
-        t.add_row(
-            ['private_ip', result.get('primaryBackendIpAddress', blank())])
+        t.add_row(['public_ip', result['primaryIpAddress'] or blank()])
+        t.add_row(['private_ip', result['primaryBackendIpAddress'] or blank()])
         t.add_row([
             'os',
             FormattedItem(
                 result['operatingSystem']['softwareLicense']
-                ['softwareDescription'].get('referenceCode', blank()),
+                ['softwareDescription']['referenceCode'] or blank(),
                 result['operatingSystem']['softwareLicense']
-                ['softwareDescription'].get('name', blank())
+                ['softwareDescription']['name'] or blank()
             )])
         t.add_row(['private_only', result['privateNetworkOnlyFlag']])
         t.add_row(['private_cpu', result['dedicatedAccountHostOnlyFlag']])
@@ -359,6 +344,8 @@ Optional:
 
   -u --userdata=DATA       User defined metadata string
   -F --userfile=FILE       Read userdata from file
+  -i --postinstall=URI     Post-install script to download
+                             (Only HTTPS executes, HTTP leaves file in /root)
   --wait=SECONDS           Block until CCI is finished provisioning for up to X
                              seconds before returning.
 """
@@ -402,29 +389,32 @@ Optional:
         data["memory"] = memory
 
         if args['--monthly']:
-            data["hourly"] = False
+            data['hourly'] = False
 
         if args.get('--os'):
-            data["os_code"] = args['--os']
+            data['os_code'] = args['--os']
 
         if args.get('--image'):
-            data["image_id"] = args['--image']
+            data['image_id'] = args['--image']
 
         if args.get('--datacenter'):
-            data["datacenter"] = args['--datacenter']
+            data['datacenter'] = args['--datacenter']
 
         if args.get('--network'):
             data['nic_speed'] = args.get('--network')
 
         if args.get('--userdata'):
             data['userdata'] = args['--userdata']
-        elif args.get('userfile'):
+        elif args.get('--userfile'):
             f = open(args['--userfile'], 'r')
             try:
                 data['userdata'] = f.read()
             finally:
                 f.close()
 
+        if args.get('--postinstall'):
+            data['post_uri'] = args.get('--postinstall')
+
         t = Table(['Item', 'cost'])
         t.align['Item'] = 'r'
         t.align['cost'] = 'r'
@@ -498,7 +488,7 @@ Optional:
     def execute(client, args):
         cci = CCIManager(client)
 
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         ready = cci.wait_for_transaction(cci_id, int(args.get('--wait') or 0))
 
         if ready:
@@ -512,6 +502,10 @@ class ReloadCCI(CLIRunnable):
 usage: sl cci reload <identifier> [options]
 
 Reload the OS on a CCI based on its current configuration
+
+Optional:
+    -i, --postinstall=URI   Post-install script to download
+                             (Only HTTPS executes, HTTP leaves file in /root)
 """
 
     action = 'reload'
@@ -520,9 +514,9 @@ Reload the OS on a CCI based on its current configuration
     @staticmethod
     def execute(client, args):
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         if args['--really'] or no_going_back(cci_id):
-            cci.reload_instance(cci_id)
+            cci.reload_instance(cci_id, args['--postinstall'])
         else:
             CLIAbort('Aborted')
 
@@ -540,7 +534,7 @@ Cancel a CCI
     @staticmethod
     def execute(client, args):
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         if args['--really'] or no_going_back(cci_id):
             cci.cancel_instance(cci_id)
         else:
@@ -581,7 +575,7 @@ Manage active CCI
     def exec_shutdown(client, args):
         vg = client['Virtual_Guest']
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         if args['--soft']:
             result = vg.powerOffSoft(id=cci_id)
         elif args['--cycle']:
@@ -595,28 +589,28 @@ Manage active CCI
     def exec_poweron(client, args):
         vg = client['Virtual_Guest']
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         return vg.powerOn(id=cci_id)
 
     @staticmethod
     def exec_pause(client, args):
         vg = client['Virtual_Guest']
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         return vg.pause(id=cci_id)
 
     @staticmethod
     def exec_resume(client, args):
         vg = client['Virtual_Guest']
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         return vg.resume(id=cci_id)
 
     @staticmethod
     def exec_reboot(client, args):
         vg = client['Virtual_Guest']
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         if args['--cycle']:
             result = vg.rebootHard(id=cci_id)
         elif args['--soft']:
@@ -652,16 +646,14 @@ Options:
 
     @staticmethod
     def exec_port(client, args):
-        vg = client['Virtual_Guest']
-        if args['--public']:
-            func = vg.setPublicNetworkInterfaceSpeed
-        elif args['--private']:
-            func = vg.setPrivateNetworkInterfaceSpeed
+        public = True
+        if args['--private']:
+            public = False
 
         cci = CCIManager(client)
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
 
-        result = func(args['--speed'], id=cci_id)
+        result = cci.change_port_speed(cci_id, public, args['--speed'])
         if result:
             return "Success"
         else:
@@ -693,7 +685,7 @@ Options:
 
     @staticmethod
     def dns_sync(client, args):
-        from SoftLayer.DNS import DNSManager, DNSZoneNotFound
+        from SoftLayer import DNSManager, DNSZoneNotFound
         dns = DNSManager(client)
         cci = CCIManager(client)
 
@@ -742,7 +734,7 @@ Options:
                     instance['fullyQualifiedDomainName'],
                     ttl=7200)
 
-        cci_id = resolve_id(cci, args.get('<identifier>'))
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
         instance = cci.get_instance(cci_id)
 
         if not instance['primaryIpAddress']:
