@@ -20,8 +20,8 @@ Available Operations:
     '<= value'  Less than or equal to value
 
 Examples:
-    sl cci list --datacenter=dal05
-    sl cci list --hostname='prod*'
+    sl hardware list --datacenter=dal05
+    sl hardware list --hostname='prod*'
     sl cci list --network=100 --cpu=2
     sl cci list --network='< 100' --cpu=2
     sl cci list --memory='>= 2048'
