@@ -16,10 +16,8 @@
 """
 from SoftLayer.consts import VERSION
 
-from API import Client, API_PUBLIC_ENDPOINT, API_PRIVATE_ENDPOINT
-from DNS import DNSManager
-from CCI import CCIManager
-from metadata import MetadataManager
+from API import *  # NOQA
+from managers import *  # NOQA
 from SoftLayer.exceptions import *  # NOQA
 
 __title__ = 'SoftLayer'
@@ -27,6 +25,5 @@ __version__ = VERSION
 __author__ = 'SoftLayer Technologies, Inc.'
 __license__ = 'The BSD License'
 __copyright__ = 'Copyright 2013 SoftLayer Technologies, Inc.'
-__all__ = ['Client', 'SoftLayerError', 'SoftLayerAPIError',
-           'API_PUBLIC_ENDPOINT', 'API_PRIVATE_ENDPOINT',
-           'DNSManager', 'CCIManager', 'MetadataManager']
+__all__ = ['Client', 'BasicAuthentication', 'SoftLayerError',
+           'SoftLayerAPIError', 'API_PUBLIC_ENDPOINT', 'API_PRIVATE_ENDPOINT']
